Docker-compose.yml parser that converts compose services into orca ServiceConfig.
Structs
- Compose
File - Top-level docker-compose.yml structure (subset we care about).
- Compose
Service - A single service in docker-compose.yml.
Enums
- Compose
Depends On - depends_on can be a list of strings or a map with condition keys.
- Compose
Env - Environment can be a list of “KEY=VALUE” strings or a map.
Functions
- parse_
compose_ file - Parse a docker-compose.yml file and return orca ServicesConfig.
- parse_
compose_ str - Parse docker-compose.yml content string into orca ServicesConfig.
- services_
to_ toml - Serialize a ServicesConfig to TOML string for writing services.toml.