Module runtime

Module runtime 

Core runtime abstraction for workload execution.

Structs

ExecResult
Result of executing a command inside a workload.
LogOpts
Options for log retrieval.
WorkloadHandle
Opaque handle returned by a runtime after creating a workload.

Traits

AsAny
Helper trait for downcasting trait objects to concrete types.
Runtime
The core runtime abstraction. Both container and Wasm runtimes implement this.

Type Aliases

LogStream
A stream of log bytes.