Core runtime abstraction for workload execution.
Structs
- Exec
Result - Result of executing a command inside a workload.
- LogOpts
- Options for log retrieval.
- Workload
Handle - Opaque handle returned by a runtime after creating a workload.
Traits
- AsAny
- Helper trait for downcasting trait objects to concrete types.
- Runtime
- The core runtime abstraction. Both container and Wasm runtimes implement this.
Type Aliases
- LogStream
- A stream of log bytes.